import { Extension, Facet, Text } from "@codemirror/state";
import { LRLanguage, LanguageSupport, LanguageDescription } from "@codemirror/language";
import { ViewPlugin, ViewUpdate, PluginValue, EditorView, Tooltip } from "@codemirror/view";
import { Diagnostic } from "@codemirror/lint";
import { CompletionContext, CompletionResult, Completion } from "@codemirror/autocomplete";
import { Position, RainDocument, LanguageServiceParams } from "@rainprotocol/rainlang";
/**
 * @public Rainlang syntax highlighter
 */
declare const RainLRLanguage: LRLanguage;
// const useLast = (values: readonly any[]) => values.reduce((_, v) => v, '');
// const documentUri = Facet.define<string, string>({ combine: useLast });
// const languageId = Facet.define<string, string>({ combine: useLast });
/**
 * @public A plugin that manages languages services processes.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 */
declare class RainLanguageServicesPlugin implements PluginValue {
    private view;
    readonly uri = "file:///untitled.rain";
    readonly languageId = "rainlang";
    version: number;
    private rainDocument;
    private textDocument;
    // constructor of the class
    constructor(view: EditorView, initOpMeta?: Uint8Array | string);
    // update the instance of TexTdocument and RainDocument
    update(update: ViewUpdate): void;
    destroy(): void;
    // update opmeta for instance of RainDocument
    updateOpmeta(opmeta: Uint8Array | string): Promise<void>;
    // handles calls for hover tooltip
    handleHoverTooltip(view: EditorView, position: Position): Promise<Tooltip | null>;
    // handles calls for code completion
    handleCompletion(context: CompletionContext, position: Position): Promise<CompletionResult | null>;
    // handles calls for docuement diagnostics
    processDiagnostics(): Promise<void>;
}
/**
 * @public Provides codemirror hover tooltips.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param doc - Codemirror Text object
 * @param rainDocument - The RainDocument
 * @param position - Position of the textDocument to get the completion items for
 * @param setting - (optional) Language service params
 * @returns Codemirror tooltip or null
 */
declare function getHoverTooltip(doc: Text, rainDocument: RainDocument, position: Position, setting?: LanguageServiceParams): Tooltip | null;
/**
 * @public Provides codemirror code completion items.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param context - Completion context
 * @param rainDocument - The RainDocument
 * @param position - Position of the textDocument to get the completion items for
 * @param setting - (optional) Language service params
 * @returns Codemirror completion result or null
 */
declare function getCompletion(context: CompletionContext, rainDocument: RainDocument, position: Position, setting?: LanguageServiceParams): CompletionResult | null;
/**
 * @public Provides codemirror diagnostics.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param doc - Codemirror Text object
 * @param rainDocument - The RainDocument
 */
declare function getDiagnostics(doc: Text, rainDocument: RainDocument): Promise<Diagnostic[]>;
/**
 * @public Converts a position to offset of a character in a codemirror text.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param doc - Codemirror text
 * @param pos - The position to convert
 * @returns - The offset of the provided position
 */
declare function posToOffset(doc: Text, pos: {
    line: number;
    character: number;
}): number | undefined;
/**
 * @public Converts offset of a character in a codemirror text to a position.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param doc - Codemirror text
 * @param offset - The offset to convert
 * @returns - The position of the provided character offset
 */
declare function offsetToPos(doc: Text, offset: number): {
    line: number;
    character: number;
};
/**
 * @public Constructs regexp out of completion items.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param completions - Codemirror completion items
 * @returns Regexp of completion items
 */
declare function prefixMatch(completions: Completion[]): RegExp[];
/**
 * @public Reduces an array to its last element
 * @param values - Values to reduce
 */
declare const useLast: (values: readonly any[]) => any;
/**
 * @public The facet used to store and retrive RainLanguageServicesPlugin instance from a EditorView.plugin()
 * after getting the facet value from the state associated with that EditoreView instance, so then can intract
 * with the plugin directly to update op meta for example.
 * This combination is provided for `rainlang` LanguageSupport class objectand not for `RainlangExtention` class.
 *
 * @example
 * ```typescript
 * // get value of the facet
 * const rainViewPlugin = editorView.state.facet(RainLanguageServicesFacet);
 *
 * // get the plugin instance
 * const rainPlugin = editorView.plugin(rainViewPlugin);
 *
 * // update op meta from plugin
 * rainPlugin.updateOpMeta("0x123...");
 * ```
 */
declare const RainLanguageServicesFacet: Facet<ViewPlugin<RainLanguageServicesPlugin>, ViewPlugin<RainLanguageServicesPlugin>>;
/**
 * @public Options to choose language services and define initial op meta
 */
type RainLanguageConfig = {
    /**
     * Provides hover tooltips
     */
    hover?: boolean;
    /**
     * Provides code completion suggestions
     */
    completion?: boolean;
    /**
     * Initial op meta to pass
     */
    initialOpMeta?: Uint8Array | string;
};
/**
 * @public Rainlang as a codemirror extention
 * @example
 * ```typescript
 * // instantiate the extension that can be directly used as codemirror extention
 * // it is instantiated so that opmeta can be updated easily
 * const rainlangExtension = new RainlangExtention(options);
 *
 * // to update op meta
 * rainlangExtension.updateOpMeta("ox456...");
 * ```
 */
declare class RainlangExtension {
    extension: Extension[];
    private plugin;
    private hover;
    private completion;
    /**
     * @public Constructor of the RainlangCodemirror class
     * @param config - options to for language services to include
     */
    /**
     * @public Constructor of the RainlangCodemirror class
     * @param config - options to for language services to include
     */
    constructor(config?: RainLanguageConfig);
    /**
     * @public Updates the op meta of this instance
     * @param opmeta - The new op meta
     */
    /**
     * @public Updates the op meta of this instance
     * @param opmeta - The new op meta
     */
    updateOpMeta(opmeta: Uint8Array | string): void;
}
/**
 * @public provides Rainlang implementation for codemirror as LanguageSupport
 * This is the standard implementation of rainlang for codemirror.
 *
 * @example
 * in order to update opmeta for it, it should be done through getting the plugin instance
 * from the EditorView with providing it the result of getting the `RainLanguageServicesFacet`
 * facet and then calling `updateOpMeta()` for it:
 * ```typescript
 * // get the `RainLanguageServicesFacet` value from editor view
 * const rainViewPlugin = view.state.facet(RainLanguageServicesFacet);
 *
 * // retrive the `RainLanguageServicesPlugin` instance
 * const rainPlugin = view.plugin(rainViewPlugin);
 *
 * // update op meta
 * rainPlugin.updateOpMeta("0x123...")
 * ```
 *
 * @param config - options to for language services to include
 * @returns rainlang LanguageSupport
 */
declare function rainlang(config?: RainLanguageConfig): LanguageSupport;
/**
 * @public Provides LanguageDescription class object for rainlang
 */
declare const RainlangDescription: LanguageDescription;
export { RainLanguageServicesPlugin, getHoverTooltip, getCompletion, getDiagnostics, posToOffset, offsetToPos, prefixMatch, useLast, RainLRLanguage, RainLanguageServicesFacet, RainLanguageConfig, RainlangExtension, rainlang, RainlangDescription };
